"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var contracts_1 = require("./contracts");
var file_element_1 = require("./file-element");
var METHOD_DELIMITER = "@";
var BRANCH_DELIMITER = "|";
var NULL_PARAM_MESSAGE = "cannot be null or undefined";
var UniqueIdConverter = /** @class */ (function () {
    function UniqueIdConverter(filename, logger) {
        if (!filename) {
            throw new Error("'filename' " + NULL_PARAM_MESSAGE);
        }
        if (!logger) {
            throw new Error("'logger' " + NULL_PARAM_MESSAGE);
        }
        this.filename = filename;
        this.logger = logger;
        this._uniqueIdsMap = {};
    }
    UniqueIdConverter.prototype.createFileElement = function (element, filename, type) {
        var startPosition = this.createElementPosition(element.position);
        var endPosition = this.createElementPosition(element.endPosition);
        return new file_element_1.FileElement(type, startPosition, endPosition, element.uniqueId, filename);
    };
    UniqueIdConverter.prototype.createElementPosition = function (position) {
        var line;
        var column;
        if (Array.isArray(position) && position.length === 2) {
            line = parseInt(position[0]);
            column = parseInt(position[1]);
        }
        else {
            throw new Error("element has no correct startPosition array, cannot resolve startPosition");
        }
        return {
            column: column,
            line: line,
        };
    };
    UniqueIdConverter.prototype.sortElements = function () {
        return this._fileElements.sort(this.comparePosition);
    };
    UniqueIdConverter.prototype.comparePosition = function (elm1, elm2) {
        if (elm1.startPosition.line < elm2.startPosition.line) {
            return -1;
        }
        if (elm1.startPosition.line > elm2.startPosition.line) {
            return 1;
        }
        if (elm1.startPosition.column < elm2.startPosition.column) {
            return -1;
        }
        if (elm1.startPosition.column > elm2.startPosition.column) {
            return 1;
        }
        return 0;
    };
    UniqueIdConverter.prototype.createElementsArray = function (file) {
        var methodsArray = [];
        var branchesArray = [];
        file.methods = file.methods || [];
        file.branches = file.branches || [];
        this.fillFileElementsArray(file.methods, methodsArray, contracts_1.ElementType.METHOD);
        this.fillFileElementsArray(file.branches, branchesArray, contracts_1.ElementType.BRANCH);
        return methodsArray.concat(branchesArray);
    };
    UniqueIdConverter.prototype.fillFileElementsArray = function (elements, array, type) {
        for (var _i = 0, elements_1 = elements; _i < elements_1.length; _i++) {
            var element = elements_1[_i];
            array.push(this.createFileElement(element, this.filename, type));
        }
    };
    UniqueIdConverter.prototype.createNewUniqueIds = function () {
        for (var i = 0; i < this._fileElements.length; i++) {
            var currentElement = this._fileElements[i];
            if (currentElement.type == contracts_1.ElementType.METHOD) {
                currentElement.uniqueId = currentElement.filename + METHOD_DELIMITER + i;
            }
            if (currentElement.type == contracts_1.ElementType.BRANCH) {
                currentElement.uniqueId = currentElement.filename + BRANCH_DELIMITER + i;
            }
            this.logger.debug("[UniqueIdConverter] uniqueId " + currentElement.uniqueIdKey + " converted to " + currentElement.uniqueId);
            this._uniqueIdsMap[currentElement.uniqueIdKey] = currentElement.uniqueId;
            if (currentElement.uniqueIdKey_decl) {
                this._uniqueIdsMap[currentElement.uniqueIdKey_decl] = currentElement.uniqueId;
            }
        }
    };
    UniqueIdConverter.prototype.process = function () {
        this.filterDuplicatesElements();
        this._fileElements = this.sortElements();
        this.removeBranchesOutOfMethods();
        this.logger.debug("sorted " + this._fileElements.length + " elements");
        this.createNewUniqueIds();
    };
    Object.defineProperty(UniqueIdConverter.prototype, "uniqueIdsMap", {
        get: function () {
            return this._uniqueIdsMap;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniqueIdConverter.prototype, "fileElements", {
        get: function () {
            return this._fileElements;
        },
        set: function (value) {
            this._fileElements = value;
        },
        enumerable: true,
        configurable: true
    });
    UniqueIdConverter.prototype.setAndInitFile = function (file) {
        this._fileElements = this.createElementsArray(file);
    };
    UniqueIdConverter.prototype.filterDuplicatesElements = function () {
        var existedKeys = {};
        this._fileElements = this._fileElements.filter(function (element) {
            if (existedKeys[element.uniqueIdKey] ||
                (element.uniqueIdKey_decl && existedKeys[element.uniqueIdKey_decl])) {
                return false;
            }
            existedKeys[element.uniqueIdKey] = true;
            if (element.uniqueIdKey_decl) {
                existedKeys[element.uniqueIdKey_decl] = true;
            }
            return true;
        });
    };
    UniqueIdConverter.prototype.removeBranchesOutOfMethods = function () {
        var allBranches = this._fileElements.filter(function (element) { return element.type === contracts_1.ElementType.BRANCH; });
        var validBranches = [];
        var allMethods = this._fileElements.filter(function (element) { return element.type === contracts_1.ElementType.METHOD; });
        var methodsIndex = 0;
        for (var _i = 0, allBranches_1 = allBranches; _i < allBranches_1.length; _i++) {
            var branch = allBranches_1[_i];
            for (var i = methodsIndex; i < allMethods.length; i++) {
                var currentMethod = allMethods[i];
                var branchPosition = this.checkBranchPosition(branch, currentMethod);
                if (branchPosition === 0) {
                    validBranches.push(branch);
                    break;
                }
                if (branchPosition === 1) {
                    methodsIndex++;
                }
            }
        }
        this._fileElements = validBranches.concat(allMethods);
        this._fileElements = this.sortElements();
    };
    /*
        this method checks if specific branch is inside specific method
        if inside method returns: 0
        if after method returns: 1
        if before method returns: -1
     */
    UniqueIdConverter.prototype.checkBranchPosition = function (branch, method) {
        if (branch.isStartsAfter(method) && branch.isEndsBefore(method)) {
            return 0;
        }
        if (branch.isStartsBefore(method) && branch.isEndsBefore(method)) {
            return -1;
        }
        if (branch.isStartsAfter(method) && branch.isEndsAfter(method)) {
            return 1;
        }
        this.logger.warn("branch started in " + branch.startPosition + " located across methods, probably some error");
        return 0;
    };
    return UniqueIdConverter;
}());
exports.UniqueIdConverter = UniqueIdConverter;
//# sourceMappingURL=unique-id-converter.js.map