"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var sl_env_vars_1 = require("../constants/sl-env-vars");
var path = require("path");
var FILE_NOT_FOUND_CODE = 'ENOENT';
var ProductionFootprintsHandler = /** @class */ (function () {
    function ProductionFootprintsHandler(cfg, logger) {
        if (!logger) {
            throw new Error("logger cannot be null or undefined");
        }
        this.validateConfig(cfg);
        this.metaData = new MetaData(cfg.buildSessionId.value, cfg.labId.value);
        this.filename = this.createFilePath(cfg);
        this.logger = logger;
        this.methodIdToHits = {};
    }
    ProductionFootprintsHandler.prototype.handleFootprints = function (footprintsPacket, callback) {
        if (!footprintsPacket) {
            this.logger.info("Handle footprints - got null or empty packet. Skip saving.");
            callback(null);
            return;
        }
        if (!footprintsPacket || !footprintsPacket.apps || !footprintsPacket.apps[0] || !footprintsPacket.apps[0].files) {
            this.logger.info("Handle footprints - no footprints data. Skip saving.");
            callback(null);
            return;
        }
        this.saveMethodsHits(footprintsPacket)
            .then(function () { return callback(null); })
            .catch(function (err) { return callback(err); });
    };
    ProductionFootprintsHandler.prototype.saveMethodsHits = function (packetToSend) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.methodIdToHits = {};
            _this.addHitsFromFootprints(packetToSend.apps[0].files);
            _this.addHitsFromFile()
                .then(function () { return _this.writeDataToFile(); })
                .then(function () { return resolve(); })
                .catch(function (err) { return reject(err); });
        });
    };
    /*
        In production listener we are in anonymous mode, and all hits will refer to single anonymous test.
        So we can there will be single array in "hits" array.
        In this inner array firs element will be always '0'(the index of test in array which contains only one - the anonymous),
        and the second element is the hits number.
     */
    ProductionFootprintsHandler.prototype.addHitsFromFootprints = function (files) {
        var _this = this;
        files.forEach(function (file) {
            file.methods.forEach(function (method) {
                _this.methodIdToHits[method.uniqueId] = method.hits[0][1];
            });
        });
    };
    ProductionFootprintsHandler.prototype.tryParseFileContent = function (fileContent) {
        try {
            var hitsData = JSON.parse(fileContent).methodsHits;
            this.logger.debug("Previous hit data parsed successfully, '%s' items loaded", Object.keys(hitsData).length);
            return hitsData;
        }
        catch (err) {
            this.logger.error("Error while parsing hits file content. ", err);
            return null;
        }
    };
    ProductionFootprintsHandler.prototype.validateConfig = function (config) {
        if (!config) {
            throw new Error("cfg cannot be null or undefined");
        }
        if (!config.labId.hasValue) {
            throw new Error("Configuration must contains labId");
        }
        if (!config.buildSessionId.hasValue) {
            throw new Error("Configuration must contains buildSessionId");
        }
    };
    ProductionFootprintsHandler.prototype.addHitsFromFile = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            fs.readFile(_this.filename, function (err, data) {
                if (err && err.code != FILE_NOT_FOUND_CODE) {
                    _this.logger.error("Error while trying to read previous hits data. ", err);
                }
                if (data != null) {
                    var hits = _this.tryParseFileContent(data.toString());
                    if (hits != null) {
                        _this.updateHits(hits);
                    }
                    else {
                        _this.logger.error("Could not load previous hits from file");
                    }
                }
                resolve();
            });
        });
    };
    ProductionFootprintsHandler.prototype.writeDataToFile = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var data = JSON.stringify(_this.toJSonObject());
            fs.writeFile(_this.filename, data, function (err) {
                if (err) {
                    _this.logger.error("Error while trying to write methods hits into file. ", err);
                    reject(err);
                }
                resolve();
            });
        });
    };
    ProductionFootprintsHandler.prototype.createFilePath = function (config) {
        var filename = this.combineArgsWithDash(config.buildSessionId.value, config.labId.value, process.pid) + ".json";
        var fileStorage = sl_env_vars_1.SlEnvVars.getFileStorage();
        if (fileStorage) {
            return path.join(fileStorage, filename);
        }
        return filename;
    };
    ProductionFootprintsHandler.prototype.combineArgsWithDash = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return args.join("-");
    };
    ProductionFootprintsHandler.prototype.updateHits = function (methodsHitFromFile) {
        var _this = this;
        Object.keys(methodsHitFromFile).forEach(function (methodId) {
            if (_this.methodIdToHits[methodId]) {
                _this.methodIdToHits[methodId] += parseInt(methodsHitFromFile[methodId]);
            }
            else {
                _this.methodIdToHits[methodId] = parseInt(methodsHitFromFile[methodId]);
            }
        });
    };
    ProductionFootprintsHandler.prototype.toJSonObject = function () {
        return {
            meta: this.metaData.toJsonObject(),
            methodsHits: this.methodIdToHits
        };
    };
    return ProductionFootprintsHandler;
}());
exports.ProductionFootprintsHandler = ProductionFootprintsHandler;
var MetaData = /** @class */ (function () {
    function MetaData(buildSessionId, labId) {
        this.buildSessionId = buildSessionId;
        this.labId = labId;
        this.createdAt = new Date().toLocaleString();
        this.pid = process.pid;
    }
    MetaData.prototype.toJsonObject = function () {
        var _this = this;
        this.lastModifiedAt = new Date().toLocaleString();
        var json = {};
        Object.keys(this).forEach(function (key) {
            json[key] = _this[key];
        });
        return json;
    };
    return MetaData;
}());
//# sourceMappingURL=production-footprints-handler.js.map